<?php declare(strict_types=1);

namespace Infomaker\Everyware\Support\Storage;

use Infomaker\Everyware\Support\Collection;
use Infomaker\Everyware\Support\Fluent;
use Infomaker\Everyware\Support\Interfaces\Arrayable;
use Infomaker\Everyware\Support\Interfaces\Jsonable;
use Infomaker\Everyware\Support\Str;

/**
 * Class CollectionModel
 * @package Infomaker\Everyware\Support\Storage
 */
class CollectionModel extends Fluent
{

    /**
     * Contains the name of the options table associated with the model
     *
     * @var string
     */
    protected $optionName;

    /**
     * Describes whether the collection can be reached across the network of sites
     *
     * @var bool
     */
    protected $global = false;

    /**
     * Fields that will be set and stored
     *
     * @var array
     */
    protected $fields = [];

    /**
     * CollectionModel constructor.
     * @param array $attributes
     */
    public function __construct($attributes = [])
    {
        $validFields = $this->getFields($attributes);
        parent::__construct($this->onSet($validFields));
    }

    /**
     * Filter attributes to only use the filed specified in the "fields property"
     *
     * @param array $attributes
     *
     * @return array
     */
    protected function getFields($attributes = []): array
    {
        $attributes = $this->getArrayableItems($attributes);

        if (empty($this->fields)) {
            return $attributes;
        }

        // Add defaults to make sure all fields are set
        $attributes = array_replace_recursive($this->fields, $attributes);

        $validFields = array_keys($this->fields);

        // Filter out field that are not specified in the "fields property"
        return array_filter($attributes, function ($key) use ($validFields) {
            return \in_array($key, $validFields, true);
        }, ARRAY_FILTER_USE_KEY);
    }

    /**
     * Results array of items from CollectionModel or Arrayable.
     *
     * @param mixed $items
     *
     * @return array
     */
    protected function getArrayableItems($items): array
    {
        if (\is_array($items)) {
            return $items;
        }

        if ($items instanceof self) {
            return $items->getAttributes();
        }

        if ($items instanceof Arrayable) {
            return $items->toArray();
        }

        if ($items instanceof Jsonable) {
            return json_decode($items->toJson(), true);
        }

        return (array)$items;
    }

    /**
     * Function that's called before fields are set as properties
     *
     * @param array $fields
     *
     * @return array
     */
    protected function onSet($fields = []): array
    {
        return $fields;
    }

    /**
     * Another way of creating and saving an collection item
     *
     * @param array $attributes
     *
     * @return self
     */
    public static function create(array $attributes = []): self
    {
        $model = new static($attributes);
        $model->save();

        return $model;
    }

    /**
     * Save current model in collection
     *
     * @return bool
     */
    public function save(): bool
    {
        return $this->newCollection()->add($this)->save();
    }

    /**
     * Connect to the CollectionDB table
     *
     * @return CollectionDB
     */
    public function newCollection(): CollectionDB
    {
        return new CollectionDB($this->optionName, $this->global);
    }

    /**
     * Find a specific value from the collection
     *
     * @param $key
     *
     * @return mixed
     */
    public static function find($key)
    {
        return static::all()->pull($key);
    }

    /**
     * Get all of the model from the database.
     *
     * @return Collection
     */
    public static function all(): Collection
    {
        $instance = new static();

        return $instance->newCollection()->all()->map(function ($item) {
            return new static($item);
        });
    }

    /**
     * Update
     *
     * @param string $key
     * @param array  $item
     *
     * @return void
     */
    public static function update($key, $item = []): void
    {
        $instance = $item instanceof self ? $item : new static($item);
        $instance->newCollection()->update($key, $instance);
    }

    /**
     * Filter items by the given key value pair.
     *
     * @param string $key
     * @param string $value
     * @param bool   $strict
     *
     * @return Collection
     */
    public static function where($key, $value, $strict = true): Collection
    {
        return static::makeCollection(static::collection()->all()->where($key, $value, $strict));
    }

    /**
     * Create a valid collection
     * Has option to use an item-property as collection key
     *
     * @param array  $items
     * @param string $use_as_key
     *
     * @return Collection
     */
    public static function makeCollection($items = [], $use_as_key = null): Collection
    {

        if ( ! $use_as_key) {
            return Collection::make($items)->map(function ($item) {
                return new static($item);
            });
        }

        $collection = [];
        foreach ($items as $key => $item) {
            if (isset($item[$use_as_key]) && Str::notEmpty($item[$use_as_key])) {
                $key = Str::slug($item[$use_as_key]);
            }
            $collection[$key] = new static($item);
        }

        return new Collection($collection);
    }

    /**
     * Handle Collection table
     *
     * @return CollectionDB
     */
    public static function collection(): CollectionDB
    {
        return (new static)->newCollection();
    }

    /**
     * Filter items by the given key value pair in depth of Collection
     *
     * @param string $key
     * @param string $value
     * @param bool   $strict
     *
     * @return Collection
     */
    public static function whereIn($key, $value, $strict = true): Collection
    {
        return static::makeCollection(static::collection()->all()->whereIn($key, $value, $strict));
    }
}
