<?php declare(strict_types=1);
/**
 * Storage handler: for handling document-database functionality by using Wordpress options table
 */

namespace Infomaker\Everyware\Support\Storage;

use Infomaker\Everyware\Support\Interfaces\Storage\WpOptionsInterface;

/**
 * Class ObjectDB
 * @package Infomaker\Everyware\Support\Storage
 */
class ObjectDB implements WpOptionsInterface
{

    /**
     * Add data to document-database
     *
     * @see   add_option()
     *
     * @param string $option
     * @param mixed  $value
     * @param bool   $autoload
     *
     * @return bool
     */
    public static function add($option, $value, $autoload = true): bool
    {
        return add_option($option, $value, '', $autoload ? 'yes' : 'no');
    }

    /**
     * Add global data to document-database
     *
     * @see   add_network_option()
     *
     * @param int    $network_id
     * @param string $option
     * @param mixed  $value
     *
     * @return bool
     */
    public static function addGlobal($network_id, $option, $value): bool
    {
        return add_network_option($network_id, $option, $value);
    }

    /**
     * Retrieve data from document-database
     *
     * @see   get_option()
     *
     * @param string $option
     * @param mixed  $default
     *
     *
     * @return mixed
     */
    public static function get($option, $default = false)
    {
        return get_option($option, $default);
    }

    /**
     * Retrieve global data from document-database
     *
     * @see   get_network_option()
     *
     * @param int    $network_id
     * @param string $option
     * @param bool   $default
     *
     ** @return mixed
     */
    public static function getGlobal($network_id, $option, $default = false)
    {
        return get_network_option($network_id, $option, $default);
    }

    /**
     * Remove data from document-database
     *
     * @see   delete_option()
     *
     * @param string $option
     *
     * @return bool
     */
    public static function remove($option): bool
    {
        return delete_option($option);
    }

    /**
     * Remove global data from document-database
     *
     * @see   delete_network_option()
     *
     * @param int    $network_id
     * @param string $option
     *
     * @return bool
     */
    public static function removeGlobal($network_id, $option): bool
    {
        return delete_network_option($network_id, $option);
    }

    /**
     * Update data on document-database
     *
     * @see   update_option()
     *
     * @param string $option
     * @param mixed  $value
     * @param bool   $autoload
     *
     * @return bool
     */
    public static function update($option, $value, $autoload = true): bool
    {
        return update_option($option, $value, $autoload ? 'yes' : 'no');
    }

    /**
     * Update global data on document-database
     *
     * @see   update_network_option()
     *
     * @param int    $network_id
     * @param string $option
     * @param mixed  $value
     *
     * @return mixed
     */
    public static function updateGlobal($network_id, $option, $value)
    {
        return update_network_option($network_id, $option, $value);
    }

    /**
     * Filters the value of an existing option.
     * Args passed to callback : {
     *
     * @var mixed      $value  Value of the option. If stored serialized, it will be
     *                       unserialized prior to being returned.
     * @var string     $option Option name.
     * }
     *
     * @param string   $option
     * @param callable $callback
     * @param int      $priority
     *
     * @return void
     */
    public static function onGet($option, callable $callback, $priority = 10): void
    {
        add_filter("option_{$option}", $callback, $priority, 2);
    }

    /**
     * Filters an option before its value is (maybe) serialized and updated.
     * Optional to choose specific option
     *
     * @see   update_option()
     * Args passed to callback : {
     * @var mixed      $value     The new, unserialized option value.
     * @var mixed      $old_value The old option value.
     * @var string     $option    Option name.
     * }
     *
     * @param string   $option
     * @param callable $callback
     * @param int      $priority
     *
     * @return void
     */
    public static function beforeUpdate($option, callable $callback, $priority = 10): void
    {
        $filterTag = $option === null ? 'pre_update_option' : "pre_update_option_{$option}";
        add_filter($filterTag, $callback, $priority, 3);
    }

    /**
     * Filters the value of an existing option before it is retrieved.
     * Passing a truthy value to the filter will short-circuit retrieving
     * the option value, returning the passed value instead.
     *
     * @see   get_option()
     * Args passed to callback : {
     * @var bool|mixed $pre_option Value to return instead of the option value.
     *                               Default false to skip it.
     * @var string     $option     Option name.
     * }
     *
     * @param string   $option
     * @param callable $callback
     * @param int      $priority
     *
     * @return void
     */
    public function beforeGet($option, callable $callback, $priority = 10): void
    {
        add_filter("pre_option_{$option}", $callback, $priority, 2);
    }
}
